/**
 * Copyright University Bordeaux - CNRS UMR 5800 
 * contributor(s) : Romain Giot (2017)
 * 
 * Romain.Giot@u-bordeaux.fr
 * 
 * Biometric Power Graph (bpg) is a computer program whose purpose is to evaluate biometric authentication
 * systems by displaying with a graph-based representation after having transformed the data.
 * 
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use, 
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * http://www.cecill.info. 
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability. 
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or 
 * data to be ensured and,  more generally, to use and operate it in the 
 * same conditions as regards security. 
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */

#include "bpg/biometricpowergraphwindow.h"
#include <QApplication>

#include <tulip/TlpQtTools.h>
#include <tulip/TlpTools.h>
#include <tulip/PluginLibraryLoader.h>
#include <tulip/PluginLoaderTxt.h>



static const char USAGE[] =
  R"(Biometric Power Graph Visualisation.

    Usage:
      powergraphViewer
      powergraphViewer --import=<import_fname> [--isDistance]
      powergraphViewer (-h | --help)
      powergraphViewer --version

    Options:
      -h --help     Show this screen.
      --version     Show version.
      --import=<import_fname>  Specify the CSV filename to import.
      --isDistance Specify the scores are distances instead of being scores by default.
)";

int main(int argc, char *argv[]) {
  QApplication app(argc, argv);

  QCoreApplication::setOrganizationName("Romain Giot/University of Bordeaux/LaBRI");
  QCoreApplication::setOrganizationDomain("http://labri.fr");
  QCoreApplication::setApplicationName("Biometric Power Graph");

  tlp::initTulipLib();
  tlp::initTulipSoftware();


  BiometricPowerGraphWindow w;
  QStringList args = app.arguments();

  QRegExp rxArgImport("--import=(.*)");
  QRegExp rxArgImport2("^--import$");
  QRegExp rxArgIsDist("--isDistance");
  QRegExp rxArgHelp("(-h)|(--help)");
  QRegExp rxArgOpenEER("--openEER");

  QString fname = "";
  bool isDistance = false;
  bool openEER = false;

     for (int i = 1; i < args.size(); ++i) {
        if (rxArgImport.indexIn(args.at(i)) != -1 ) {
            fname = rxArgImport.cap(1);
	}
	else if (rxArgImport2.indexIn(args.at(i)) != -1 ) {
	    ++i;
	    fname = args[i];
	}
	else if (rxArgIsDist.indexIn(args.at(i)) != -1) {
	    isDistance = true;
        }
        else if (rxArgHelp.indexIn(args.at(i)) != -1 ) {
		std::cout << USAGE << std::endl;
	    return 0;
        }
        else if (rxArgOpenEER.indexIn(args.at(i)) != -1 ) {
	     openEER = true;
        }
        else {
            qDebug() << "Uknown arg:" << args.at(i);
        }
    }

   if (fname.size() >0) {
	   w.importCSV(fname, !isDistance, "", openEER);
   }
   w.show();

  return app.exec();
}
