/**
 * Copyright University Bordeaux - CNRS UMR 5800 
 * contributor(s) : Romain Giot (2017)
 * 
 * Romain.Giot@u-bordeaux.fr
 * 
 * Biometric Power Graph (bpg) is a computer program whose purpose is to evaluate biometric authentication
 * systems by displaying with a graph-based representation after having transformed the data.
 * 
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use, 
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * http://www.cecill.info. 
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability. 
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or 
 * data to be ensured and,  more generally, to use and operate it in the 
 * same conditions as regards security. 
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */

#ifndef BIOMETRICDATABASEMODEL_H
#define BIOMETRICDATABASEMODEL_H

#include <vector>
#include <array>

#include <QAbstractTableModel>
#include <QColor>

#include "bpg/BPGConfig.h"

/**
 * @brief The BiometricDatabaseModel class
 *
 * Allow to represents the biometric database of score as a QT model in order to use QT facilities to manipulate it
 */
class BPG_UTILS_SCOPE BiometricDatabaseModel : public QAbstractTableModel
{
private:
  /**
     * @brief NB_COLUMNS Number of columns of the table
     */
  static constexpr int NB_COLUMNS = 5;

public:
  using InnerType = std::vector< std::array< double, NB_COLUMNS > >;

private:
  InnerType _data;
  bool _isScore;

public:
  BiometricDatabaseModel(QObject *parent = 0);
  ~BiometricDatabaseModel();

  static const QColor TrueMatch;
  static const QColor FalseNonMatch;
  static const QColor TrueNonMatch;
  static const QColor FalseMatch;

  // Mandatory methods to implement
  int rowCount(const QModelIndex &parent = QModelIndex()) const override;

  int columnCount(const QModelIndex &parent = QModelIndex()) const override;

  QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const override;

  // Good code implement that
  QVariant headerData(int section, Qt::Orientation orientation, int role = Qt::DisplayRole) const override;


  void updateDataTable(const InnerType &data, bool isScore);
  void updateUniqueDecisionThreshold(const double thr);
};

BPG_UTILS_SCOPE BiometricDatabaseModel &getUniqueQtBiometricDataModel();

#endif // BIOMETRICDATABASEMODEL_H
