/**
 * Copyright University Bordeaux - CNRS UMR 5800 
 * contributor(s) : Romain Giot (2017)
 * 
 * Romain.Giot@u-bordeaux.fr
 * 
 * Biometric Power Graph (bpg) is a computer program whose purpose is to evaluate biometric authentication
 * systems by displaying with a graph-based representation after having transformed the data.
 * 
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use, 
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * http://www.cecill.info. 
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability. 
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or 
 * data to be ensured and,  more generally, to use and operate it in the 
 * same conditions as regards security. 
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */

#ifndef __BIO_POWERGRAPH_COMMON_H__
#define __BIO_POWERGRAPH_COMMON_H__


// XXX TODO find the good way to do that
#define USE_TLP_ITERATOR 0

#define MORE_PARALLEL_STUFF 1
#include "tulip/TulipRelease.h"

#include <tulip/ForEach.h>
#include <tulip/Graph.h>
#include <tulip/IntegerProperty.h>
#include <tulip/DoubleProperty.h>
#include <tulip/GraphProperty.h>

#include "utils.h"
#include "bpg/BPGLookup.h"

#include <cassert>


const std::string LAYER_USERS = "usersLayer";
const std::string LAYER_INDIVIDUALS = LAYER_USERS;
const std::string LAYER_SAMPLES = "samplesLayer";
const std::string LAYER_QUERY2GALLERY = "query2galleryLayer";
const std::string LAYER_SAMPLE2OWNER = "sample2ownerLayer";
const std::string LAYER_DISPLAY = "display";
const std::string LAYER_ROC_CURVE = "ROC";

const std::string PROP_SCORE = "score";
const std::string PROP_USER = "userId";
const std::string PROP_SAMPLE = "userSampleId";
const std::string PROP_USER_GRAPH = "userSubGraph";
const std::string PROP_IS_QUERY_ACCEPTED = "queryAcceptedAtSelectedThreshold";


const std::string PROP_IS_GOAT = "isGoat";
const std::string PROP_IS_LAMB = "isLamb";
const std::string PROP_IS_WOLF = "isWolf";
const std::string PROP_IS_SHEEP = "isSheep";

const std::string PROP_IS_THRESHOLDED_GOAT = "isThresholdedGoat";
const std::string PROP_IS_THRESHOLDED_LAMB = "isThresholdedLamb";
const std::string PROP_IS_THRESHOLDED_WOLF = "isThresholdedWolf";
const std::string PROP_IS_THRESHOLDED_SHEEP = "isThresholdedSheep";

const std::string PROP_MEAN_SELF_SCORE = "intraScore";
const std::string PROP_MEAN_OUT_SCORE = "interScore1";
const std::string PROP_MEAN_IN_SCORE = "interScore2";
const std::string PROP_MEAN_MAX_PAIR_IN_SCORE = "lambScore";
const std::string PROP_MEAN_MAX_PAIR_OUT_SCORE = "wolfScore";

const std::string PROP_IN_FMR = "FMR";
const std::string PROP_OUT_FMR = "FMR2";
const std::string PROP_FNMR = "FNMR";
const std::string PROP_FMR = "FMR";
const std::string PROP_THR = "THR";


const std::string PROP_IN_FMR_CLUSTER = "FMR_cluster";
const std::string PROP_OUT_FMR_CLUSTER = "FMR2_cluster";
const std::string PROP_FNMR_CLUSTER = "FNMR_cluster";


const std::string PROP_CLUSTER_VALUE = "zooScore";

const std::string ATTR_EER = "EER_value";
const std::string ATTR_THR = "EER_threshold";
const std::string ATTR_NB_INTRA = "Nb intra scores";
const std::string ATTR_NB_INTER = "Nb inter scores";
const std::string ATTR_NB_DISTINCT_SCORES = "Nb scores";

const std::string CSV_GALLERY_USER = "UserGallery";
const std::string CSV_PROBE_USER = "UserProbe";
const std::string CSV_PROBE_USER_SAMPLE = "SampleProbe";
const std::string CSV_SCORE = "Score";


inline const BPGLookup& getLut(const tlp::Graph* const g) {
   BPGLookup * ptrLut = nullptr;

   bool res = g->getRoot()->getAttribute("LUT", ptrLut);
   assert(res);
   assert(nullptr!=ptrLut);

   return *ptrLut;
}


inline double getComparisonScore(tlp::Graph * const root, tlp::edge e) {
  assert(e.isValid());
  assert(root == root->getRoot());
  assert(root->getSubGraph(LAYER_QUERY2GALLERY)->isElement(e));

  auto const prop = root->getProperty< tlp::DoubleProperty >(PROP_SCORE);
  return prop->getEdgeValue(e);
}

inline const std::vector< tlp::node > &getUsers(const tlp::Graph *const root) {
  return getLut(root).getUsers();
}




[[deprecated]]
inline const std::vector< tlp::node > &getUsersVector(tlp::Graph *const root) {
  return getUsers(root);
}


inline bool isIndividual(tlp::Graph *const root, const tlp::node n) {
  assert(root->getRoot() == root);

  const auto & users = getUsers(root);
  const bool res = std::find(users.begin(), users.end(), n) != users.end();

#ifndef NDEBUG
  const bool res2 = root->getSubGraph(LAYER_USERS)->isElement(n);
  assert(res == res2);
#endif

  return res;
}

[[deprecated]] inline bool isUser(tlp::Graph *root, tlp::node n) {
  return isIndividual(root, n);
}


inline tlp::node getEERRocNode(tlp::Graph *root) {
  assert(root);
  assert(root->getRoot() == root);


#if 0
  tlp::Graph *roc = root->getSubGraph(LAYER_ROC_CURVE);
  assert(roc);



  // Retrieve EER threshold
  double threshold = 0;
  bool threshold_obtained = root->getAttribute(ATTR_THR, threshold);
  if (!threshold_obtained) {
    assert(false);
  }

  tlp::DoubleProperty *const propThreshold = roc->getProperty< tlp::DoubleProperty >(PROP_THR);

  for (const tlp::node point : roc->nodes()) {
    if (propThreshold->getNodeValue(point) == threshold) {
      return point;
    }
  }

  assert(false);
  return {};
#endif

  unsigned int eerid = 0; // BUG Need to put maximum limit
  root->getAttribute("EER_node_id", eerid);
  return tlp::node(eerid);
}


inline double getFNMR(tlp::Graph *graph, tlp::node user) {
  return graph->getProperty< tlp::DoubleProperty >(PROP_FNMR)->getNodeValue(user);
}
inline double getFMR(tlp::Graph *graph, tlp::node user) {
  return graph->getProperty< tlp::DoubleProperty >(PROP_FMR)->getNodeValue(user);
}


inline bool isSample(tlp::Graph *root, const tlp::node n) {
  assert(root->getRoot() == root);
  const auto & samples = getLut(root).getSamples();
  const bool res = std::find(samples.begin(), samples.end(), n) != samples.end();

#ifndef NDEBUG
  const bool res2 = root->getSubGraph(LAYER_SAMPLES)->isElement(n);
  assert (res==res2);
#endif
  return res;
}


template < typename T >
class VectorWrappedIterator : public tlp::Iterator< T >
{
public:
  VectorWrappedIterator(const std::vector< T > &v)
  : v{v}, position{this->v.begin()} { // XXX Pay attention to use the right iterator ...
  }


  T next() override {
    assert(position != v.end());

    auto value = *position;
    position = std::next(position);
    return value;
  }

  bool hasNext() override {
    return v.end() != position;
  }

private:
  const std::vector< T > v;
  decltype(v.begin()) position;
};
/*
[[deprecated]]
template < typename T >
class FilteredIterator : public tlp::Iterator< T >
{
public:
  FilteredIterator(tlp::Iterator< T > *iter)
  : iter(iter), _has_element(false) {
      if (nullptr != iter && iter->hasNext()) skip_wrongs();
  }

  virtual ~FilteredIterator() {
    delete this->iter;
  }

  virtual T next() override {
      assert(_has_element);
    T current = this->_next;
    this->skip_wrongs();
    return current;
  }

  virtual bool hasNext() override {
    return _has_element;
  }

protected:
  void skip_wrongs() {
      std::cout << "skip_wrong: ";

    bool filter = true;
    _has_element = false;
    while (filter && this->iter->hasNext()) {
      this->_toFilter = this->iter->next();
      std::cout << "Need to filter" << _toFilter << std::endl;
      if (this->is_valid()) {
          std::cout << " keep" << std::endl;
        this->_next = std::move(this->_toFilter);
        this->_has_element = true;
        filter = false;
      }
      else {
          std::cout << " do not keep" << std::endl;
      }
    }

  }



  // virtual bool is_valid (T value) = 0; // XXX Does not work at runtime ...
  virtual bool is_valid() = 0; // The information to filter is in _toFilter ...

  tlp::Iterator< tlp::edge > * const iter;
  T _next;
  T _toFilter; // XXX need to understand why i NEED to do like that ...
  bool _has_element;
};


[[deprecated]]
class IntraInterIterator : public FilteredIterator< tlp::edge >
{
public:
  IntraInterIterator(tlp::Iterator< tlp::edge > *iter, tlp::Graph * g, const tlp::node user, const bool intra)
  : FilteredIterator{iter}, g{g}, user{user}, intra{intra} {
  }

protected:
  bool is_valid() override {
      std::cout << "is_valid?"; 
    assert(nullptr !=g);
    // Property which contains the id of the user
    auto userProp = g->getProperty< tlp::IntegerProperty >(PROP_USER);
    assert(nullptr != userProp);

    assert(userProp->getNodeValue(g->source(_toFilter)) > 0);
    assert(userProp->getNodeValue(g->target(_toFilter)) > 0);

    std::cout << g->source(_toFilter) << "/" << g->target(_toFilter);

    // Verification if the properties are similar
    const bool intra_comparison =
      userProp->getNodeValue(g->source(_toFilter)) ==
      userProp->getNodeValue(g->target(_toFilter));

    return intra ? intra_comparison : !intra_comparison;
  }

  tlp::Graph * g;
  const tlp::node user;
  const bool intra;
};

class ThresholdFilter : public FilteredIterator< tlp::edge >
{
public:
  ThresholdFilter(tlp::Iterator< tlp::edge > *iter, tlp::DoubleProperty *prop, double threshold)
  : FilteredIterator(iter), prop(prop), threshold(threshold) {
  }


protected:
  bool is_valid() override {
    return prop->getEdgeValue(_toFilter) >= threshold;
  }


  tlp::DoubleProperty *prop;
  const double threshold;
};

// XXX does not seem to work :(
template < typename Type, typename Filter >
class GeneralFilter : public FilteredIterator< Type >
{
public:
  GeneralFilter(tlp::Iterator< Type > *iter, const Filter filter)
  : FilteredIterator< Type >(iter), filter(filter) {
  }

protected:
  bool is_valid() override {
    return filter(this->_toFilter);
  }

  const Filter filter;
};
*/


inline size_t getNumberOfUsers(tlp::Graph *root) {
  return getUsers(root).size();
}

inline const std::vector< tlp::node > &getSamples(const tlp::Graph *root) {
  assert(root == root->getRoot());
  return getLut(root).getSamples();
}


inline size_t getNumberOfSamples(tlp::Graph *root) {
  assert(root == root->getRoot());
  return getSamples(root).size();
}

inline tlp::node getOwner(tlp::Graph *root, tlp::node sample) {
  assert(root == root->getRoot());
  assert(isSample(root, sample));

  tlp::Graph *ownerLayer = root->getSubGraph(LAYER_SAMPLE2OWNER);
  assert(ownerLayer->outdeg(sample) == 1);

  tlp::node res = ownerLayer->getOutNode(sample, 1);
  assert(res.isValid());
  assert(isIndividual(root, res));

  return res;
}

inline const std::vector< tlp::node> & getSamples(tlp::Graph * root, tlp::node const user) {
  return getLut(root).getSamplesFor(user);
}




inline size_t getNumberOfSamplesForIndividual(tlp::Graph * root, tlp::node indiv) {
	assert(root->getSubGraph(LAYER_USERS)->getSubGraph("clean")->isElement(indiv));
  return getSamples(root, indiv).size();
}


inline tlp::Graph *getGraphForUser(tlp::Graph *root, tlp::node user) {
  assert(user.isValid());
  assert(root == root->getRoot());
  assert(root->getSubGraph(LAYER_USERS)->getSubGraph("clean")->isElement(user));

  auto const prop = root->getProperty< tlp::GraphProperty >(PROP_USER_GRAPH);
  tlp::Graph *const sg = prop->getNodeValue(user);
  assert(nullptr != sg);
  assert(root->getProperty< tlp::IntegerProperty >(PROP_USER)->getNodeValue(sg->getOneNode()) == root->getProperty< tlp::IntegerProperty >(PROP_USER)->getNodeValue(user));

  return sg;
}

inline tlp::node getUserOfSample(tlp::Graph *root, tlp::node sample) {
  assert(sample.isValid());
  assert(root == root->getRoot());

  tlp::Graph *sg = root->getSubGraph(LAYER_SAMPLE2OWNER);

  tlp::node user;
#ifndef NDEBUG
  size_t nb = 0;
#endif
  forEach (user, sg->getOutNodes(sample)) {
#ifndef NDEBUG
    ++nb;
#endif
  }
  assert(1 == nb && "By construction, the number of nodes is 1");


  assert(user.isValid());
  return user;
}

/**
 * @todo improve this very inefficient implementation since buggy? filter has been removed
 */
inline const std::vector< tlp::edge >& getIntraComparisonsToUser(tlp::Graph *const root, const tlp::node user) {
  return getLut(root).getIntraComparisonOfUser(root, user);
}

/**
 * Search the score edges which go to this user
 */
inline const std::vector< tlp::edge >& getInterComparisonsToUser(tlp::Graph *root, const tlp::node user) {
  return getLut(root).getInterComparisonVectorToUser(root, user);
}


inline bool isComparisonScoreAccepted(const double score, const double thr) {
  return score >= thr;
}

inline const std::vector< tlp::edge >& getAcceptedComparisonVector(tlp::Graph *root, const double threshold) {
  return getLut(root).getAcceptedComparisons(root, threshold);
}

/**
 * Returns the comparison when impersonator is compared to user
 */
inline tlp::Iterator< tlp::edge > *getComparisons(tlp::Graph *const root, tlp::node user, tlp::node impersonator) {
  assert(root == root->getRoot());

#ifndef NDEBUG
  auto const usersSg = root->getSubGraph(LAYER_USERS)->getSubGraph("clean");
  assert(usersSg->isElement(user));
  assert(usersSg->isElement(impersonator));
#endif


  auto const query2gallery = root->getSubGraph(LAYER_QUERY2GALLERY);
  auto const sample2owner = root->getSubGraph(LAYER_SAMPLE2OWNER);

  tlp::node sample;
  std::vector< tlp::edge > edges;
  // loop over the samples of the impersonator
  forEach (sample, sample2owner->getInNodes(impersonator)) {
    tlp::edge currentEdge;
    // loop over the edges of the current sample
    forEach (currentEdge, query2gallery->getOutEdges(sample)) {
      assert(query2gallery->source(currentEdge) == sample);

      // and only store the edges of the chosen user
      const tlp::node currentUserNode = query2gallery->target(currentEdge);
      if (currentUserNode == user) {
        edges.push_back(currentEdge);
      }
    }
  }

  return new VectorWrappedIterator< tlp::edge >(edges);
}


inline std::tuple< double, double > computeFNMRandFMRforThreshold(tlp::Graph *const root, const double thr) {
  auto const userProp = root->getProperty< tlp::IntegerProperty >(PROP_USER);

  int nbComparisons = 0;
  int nbFM = 0;
  int nbFNM = 0;

  for (tlp::edge cmp : root->getSubGraph(LAYER_QUERY2GALLERY)->edges()) {
    const double score = getComparisonScore(root, cmp);
    const bool isIntra = userProp->getNodeValue(root->source(cmp)) == userProp->getNodeValue(root->target(cmp));
    const bool isAccepted = isComparisonScoreAccepted(score, thr);

    if (isIntra && !isAccepted)
      ++nbFNM;
    if (!isIntra && isAccepted)
      ++nbFM;
    ++nbComparisons;
  }

  return {
    static_cast< double >(nbFNM) / static_cast< double >(nbComparisons),
    static_cast< double >(nbFM) / static_cast< double >(nbComparisons)};
}

/**
 * Search the score edges which come from this user
 * @todo All over methods really iterate ; it would be nice to do the same thing here
 */
inline tlp::Iterator< tlp::edge > *getInterComparisonIterator2(tlp::Graph *const root, tlp::node user) {
  assert(root == root->getRoot());

  auto const sg = root->getSubGraph(LAYER_QUERY2GALLERY);
  assert(sg->isElement(user));

  auto const userProp = root->getProperty< tlp::IntegerProperty >(PROP_USER);

  const std::vector< tlp::edge > &initialEdges = sg->edges();
  std::vector< tlp::edge > filteredEdges;

  const int requiredUser_id = userProp->getNodeValue(user);

  std::copy_if(
    initialEdges.begin(),
    initialEdges.end(),
    std::back_inserter(filteredEdges),
    [&](const tlp::edge &_toFilter) {
      const int galleryUser_id = userProp->getNodeValue(root->target(_toFilter));
      const int probeUser_id = userProp->getNodeValue(root->source(_toFilter));

      return galleryUser_id != probeUser_id && probeUser_id == requiredUser_id;
    });

  return new VectorWrappedIterator< tlp::edge >(filteredEdges);
}


// XXX avoid to use this very slow method in O(N) It would be better to use a specific structure
inline tlp::node getNodeForIndividual(const tlp::Graph *g, int id) {
	tlp::Graph * const indivs = g->getSubGraph(LAYER_INDIVIDUALS);
	const auto propId = indivs->getProperty<tlp::IntegerProperty>(PROP_USER);

	for(tlp::node individual: indivs->nodes()){
		if (propId->getNodeValue(individual) == id) return individual;
	}

	assert(false);
	return tlp::node();
}



inline tlp::node getNodeSampleOfIndividualForId(tlp::Graph * const g, const tlp::node indiv, const int smplid) {
	tlp::node sample;
	const auto propId = g->getProperty<tlp::IntegerProperty>(PROP_SAMPLE);

	for(tlp::node sample: getSamples(g, indiv)) {
		if (propId->getNodeValue(sample) == smplid) return sample;
	}

	assert(false);
	return tlp::node();
}

#endif
