/**
 * Copyright University Bordeaux - CNRS UMR 5800 
 * contributor(s) : Romain Giot (2017)
 * 
 * Romain.Giot@u-bordeaux.fr
 * 
 * Biometric Power Graph (bpg) is a computer program whose purpose is to evaluate biometric authentication
 * systems by displaying with a graph-based representation after having transformed the data.
 * 
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use, 
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * http://www.cecill.info. 
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability. 
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or 
 * data to be ensured and,  more generally, to use and operate it in the 
 * same conditions as regards security. 
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */

#include <tulip/Coord.h>
#include <utility>


// http://stackoverflow.com/questions/15311018/c-procedure-for-determining-whether-two-segments-intersect
inline double CCW(const tlp::Coord &a, const tlp::Coord &b, const tlp::Coord &c) {
  return (b.x() - a.x()) * (c.y() - a.y()) - (b.y() - a.y()) * (c.x() - a.x());
}


inline bool segmentsIntersect(const std::pair< tlp::Coord, tlp::Coord > &line1, const std::pair< tlp::Coord, tlp::Coord > &line2) {
  const tlp::Coord &a = std::get< 0 >(line1);
  const tlp::Coord &b = std::get< 1 >(line1);
  const tlp::Coord &c = std::get< 0 >(line2);
  const tlp::Coord &d = std::get< 1 >(line2);
  return (CCW(a, b, c) * CCW(a, b, d) < 0 && CCW(c, d, b) * CCW(c, d, a) < 0);
}

inline bool segmentsIntersect(const tlp::Coord line1[2], const tlp::Coord line2[2]) {
  const tlp::Coord &a = line1[0];
  const tlp::Coord &b = line1[1];
  const tlp::Coord &c = line2[0];
  const tlp::Coord &d = line2[1];
  return (CCW(a, b, c) * CCW(a, b, d) < 0 && CCW(c, d, b) * CCW(c, d, a) < 0);
}
