#include "bpg/BPGLookup.h"
#include "bpg/common.h"

#include "omp.h"

const std::vector< tlp::node > & BPGLookup::getUsers()  const {
  return _users;
}

const std::vector< tlp::node > & BPGLookup::getSamples()  const {
  return _samples;
}

const std::vector< tlp::node > & BPGLookup::getSamplesFor(const tlp::node user)  const {
  return _userNode2samplesNodes.at(user);
}

// TODO Improve implementation by not going throw ALL edges
const std::vector< tlp::edge > & BPGLookup::getIntraComparisonOfUser(tlp::Graph *const root, const tlp::node user) const {
  assert(user.isValid());
  assert(root == root->getRoot());

   auto iter = _user2IntraEdges.end();
#pragma omp critical(_user2IntraEdges)
    iter = _user2IntraEdges.find(user);
  if (iter != _user2IntraEdges.end()) return *iter->second;

  auto const sg = root->getSubGraph(LAYER_QUERY2GALLERY);
  assert(sg);
  assert(sg->isElement(user));

  auto const userProp = root->getProperty< tlp::IntegerProperty >(PROP_USER);

  const std::vector< tlp::edge > &initialEdges = sg->edges();
  std::vector< tlp::edge > *filteredEdges = new std::vector< tlp::edge >;
  const int requiredUser_id = userProp->getNodeValue(user);

  std::copy_if(
    initialEdges.begin(),
    initialEdges.end(),
    std::back_inserter(*filteredEdges),
    [&](const tlp::edge &_toFilter) {
      const int galleryUser_id = userProp->getNodeValue(root->target(_toFilter));
      const int probeUser_id = userProp->getNodeValue(root->source(_toFilter));

      return galleryUser_id == probeUser_id && galleryUser_id == requiredUser_id;
    });

#pragma omp critical(_user2IntraEdges)
 _user2IntraEdges.insert({user, filteredEdges});
 return *_user2IntraEdges.at(user);
}



// TODO Improve implementation by not going throw ALL edges
const std::vector< tlp::edge > & BPGLookup::getInterComparisonVectorToUser(tlp::Graph *const root, const tlp::node user) const {
  assert(user.isValid());
  assert(root == root->getRoot());

   auto iter = _user2InterEdges.end();
#pragma omp critical(_user2InterEdges)
    iter = _user2InterEdges.find(user);
  if (iter != _user2InterEdges.end()) return *iter->second;

  auto const sg = root->getSubGraph(LAYER_QUERY2GALLERY);
  assert(sg);
  assert(sg->isElement(user));

  auto const userProp = root->getProperty< tlp::IntegerProperty >(PROP_USER);

  const std::vector< tlp::edge > &initialEdges = sg->edges();
  std::vector< tlp::edge > *filteredEdges = new std::vector< tlp::edge>;
  const int requiredUser_id = userProp->getNodeValue(user);

  std::copy_if(
    initialEdges.begin(),
    initialEdges.end(),
    std::back_inserter(*filteredEdges),
    [&](const tlp::edge &_toFilter) {
      const int galleryUser_id = userProp->getNodeValue(root->target(_toFilter));
      const int probeUser_id = userProp->getNodeValue(root->source(_toFilter));

      return galleryUser_id != probeUser_id && galleryUser_id == requiredUser_id;
    });

#pragma omp critical(_user2InterEdges)
 _user2InterEdges.insert({user, filteredEdges});
 return *_user2InterEdges.at(user);
}

const std::vector< tlp::edge >& BPGLookup::getAcceptedComparisons(tlp::Graph *root, const double threshold) const {

  assert(root == root->getRoot());

   auto iter = _threshold2AcceptedEdges.end();
#pragma omp critical(_threshold2AcceptedEdges)
    iter = _threshold2AcceptedEdges.find(threshold);
  if (iter != _threshold2AcceptedEdges.end()) return *iter->second;


  auto const sg = root->getSubGraph(LAYER_QUERY2GALLERY);
  auto const score = root->getProperty< tlp::DoubleProperty >(PROP_SCORE);

  const std::vector< tlp::edge > &initialEdges = sg->edges();
  std::vector< tlp::edge > * filteredEdges = new std::vector<tlp::edge>;

  std::copy_if(
    initialEdges.begin(),
    initialEdges.end(),
    std::back_inserter(*filteredEdges),
    [&](const tlp::edge &_toFilter) {
      return isComparisonScoreAccepted(score->getEdgeValue(_toFilter), threshold);
    });


#pragma omp critical(_threshold2AcceptedEdges)
  _threshold2AcceptedEdges.insert({threshold, filteredEdges});
  return *_threshold2AcceptedEdges.at(threshold);
}



#ifndef NDEBUG
  void BPGLookup::checkup(tlp::Graph *g) {
    const size_t nbSamplesLUT = getSamples().size();
    const size_t nbIndivLUT = getUsers().size();

    const size_t nbSamplesGraph = g->getSubGraph(LAYER_SAMPLES)->numberOfNodes();
    const size_t nbIndivGraph = g->getSubGraph(LAYER_USERS)->numberOfNodes();

    assert(nbSamplesLUT == nbSamplesGraph);
    assert(nbIndivLUT == nbIndivGraph);
  }
#endif


const tlp::node BPGLookup::getUser(int _id) const {
  return _userId2node.at(_id);
}
