#include "bpg/ExecutionMonitor.h"

#include <string>
#include <vector>
#include <chrono>
#include <iostream>

using duration = std::chrono::milliseconds;

struct Event {
	duration _duration;
	decltype(std::chrono::system_clock::now()) _start;
	std::string _name;
};

namespace execution_monitor {
	std::string last_event;
	decltype(std::chrono::system_clock::now()) last_start;
	std::vector<Event> events;

	void start(const std::string event) {
		if (last_event.size() != 0) {
			stop();
		}
		last_event = event;
		last_start = std::chrono::system_clock::now();
	}

	/**
	 * Stop the current event
	 */
	void stop() {
		Event e;
		e._duration = std::chrono::duration_cast<duration>(std::chrono::system_clock::now() - last_start);
		e._start = last_start;
		e._name = last_event;
		events.push_back(e);
		last_event = "";
	}

	void print() {
		for(Event e: events) {
			std::cout << "[TIMING] ==>" << e._name << "\t" << e._duration.count() << "<==" << std::endl;
		}
		events.clear();// Ensure no event is displayed 2 times
	}
}

