/**
 *
 * This file is part of Tulip (www.tulip-software.org)
 *
 * Authors: David Auber and the Tulip development Team
 * from LaBRI, University of Bordeaux
 *
 * Tulip is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *
 * Tulip is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 */
#include <tulip/GlXMLTools.h>

#include <tulip/DrawingTools.h>
#include <tulip/DrawingTools.h>

#include "bpg/GlHalfCircle.h"

using namespace std;

const double angleDelta = 3.14 / 2;

namespace tlp {

//===================================================
GlHalfCircle::GlHalfCircle(const Coord &center, float radius,
                           const Color &outlineColor,
                           const Color &fillColor,
                           bool filled, bool outlined,
                           float startAngle,
                           unsigned int segments)
: pa(nullptr),
  pb(nullptr),
  position(center),
  size(Size(radius, radius, 0)),
  numberOfSides{segments},
  startAngle(startAngle + angleDelta),
  outlineColor{outlineColor},
  fillColor{fillColor},
  filled{filled},
  outlined{outlined} /*,
  numberOfSides(segments),
  GlRegularPolygon(center,Size(radius,radius,0),segments,fillColor,outlineColor,filled,outlined)*/ {
  assert(segments <= 256);

  set(position, radius, startAngle + angleDelta);
}


GlHalfCircle::~GlHalfCircle() {
  if (pa != nullptr) {
    delete pa;
  }
  if (pb != nullptr) {
    delete pb;
  }
}

//===========================================================
void GlHalfCircle::set(const Coord &center, float radius, float startAngle) {
  this->position = center;
  this->size = Size(radius, radius, 0);
  this->startAngle = startAngle;

  if (pa != nullptr) {
    delete pa;
  }
  if (pb != nullptr) {
    delete pb;
  }
  pa = pb = nullptr;

  const auto main_poly = computeRegularPolygon(numberOfSides, position, size, startAngle);
  const auto nb_points = main_poly.size();
  const auto nb_points_a = nb_points / 2;
  // const auto nb_points_b = nb_points - nb_points_a;
  //


  pa = new GlPolygon(
    std::vector< Coord >{main_poly.begin(), main_poly.begin() + nb_points_a},
    std::vector< Color >{fillColor, 1 /*, nb_points_a*/},
    std::vector< Color >{fillColor, 1 /*, nb_points_a*/},
    filled,
    true /*outlined*/,
    "",
    outlineSize);


  pb = new GlPolygon(
    std::vector< Coord >{main_poly.begin() + nb_points_a, main_poly.end()},
    std::vector< Color >{outlineColor, 1 /*, nb_points_b*/},
    std::vector< Color >{outlineColor, 1 /*, nb_points_b*/},
    filled,
    true /*outlined*/,
    "",
    outlineSize);
}

void GlHalfCircle::setGenuineColor(const Color &color) {
  if (pa) {
    pa->setFillColor(color);
  }
  fillColor = color;
}

void GlHalfCircle::setImpostorColor(const Color &color) {
  if (pb) {
    pb->setFillColor(color);
  }
  outlineColor = color;
}


void GlHalfCircle::setOutlineMode(const bool /*outlined*/) {
  this->outlined = false /*outlined*/;
}

void GlHalfCircle::setOutlineSize(float size) {
  outlineSize = size;
}


void GlHalfCircle::setLightingMode(const bool lighting) {
  if (pa) {
    pa->setLightingMode(lighting);
  }
  if (pb) {
    pb->setLightingMode(lighting);
  }
}


void GlHalfCircle::draw(float lod, Camera *c) {
  pa->draw(lod, c);
  pb->draw(lod, c);
}

//===========================================================
void GlHalfCircle::getXML(string &outString) {

  GlXMLTools::createProperty(outString, "type", "GlHalfCircle", "GlEntity");

  // GlPolygon::getXMLOnlyData(outString);
}


void GlHalfCircle::setWithXML(const string & /*inString*/, unsigned int & /*currentPosition*/) {
}
}
