/**
 * Copyright University Bordeaux - CNRS UMR 5800 
 * contributor(s) : Romain Giot (2017)
 * 
 * Romain.Giot@u-bordeaux.fr
 * 
 * Biometric Power Graph (bpg) is a computer program whose purpose is to evaluate biometric authentication
 * systems by displaying with a graph-based representation after having transformed the data.
 * 
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use, 
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * http://www.cecill.info. 
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability. 
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or 
 * data to be ensured and,  more generally, to use and operate it in the 
 * same conditions as regards security. 
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */


#ifndef BIOMETRIC_POWER_NODEGLYPH_H
#define BIOMETRIC_POWER_NODEGLYPH_H

#include <tulip/Glyph.h>
#include <tulip/GlRect.h>
#include <tulip/GlLabel.h>
#include <tulip/GlPolyQuad.h>
#include <tulip/GlQuad.h>
#include <tulip/GlTextureManager.h>
#include <tulip/GlGraphRenderingParameters.h>
#include <tulip/ColorProperty.h>
#include <tulip/GlGraphInputData.h>
#include <tulip/TlpTools.h>
#include <tulip/TulipViewSettings.h>
#include <array>


#include "bpg/common.h"

using namespace tlp;
using namespace std;

// XXX When updating it, do not forget to upate ClusterColorLegend

const auto FNMR_COLOR_START = tlp::Color::Gray;
const auto FNMR_COLOR_STOP  = tlp::Color::Green;
static const std::array< tlp::Color, 4 > FNMR_COLORS_LUT{{ FNMR_COLOR_START,
                                                {
                                                    static_cast<unsigned char>( static_cast<double>(FNMR_COLOR_START.r())*2.0/3.0 + static_cast<double>(FNMR_COLOR_STOP.r()*1.0/3.0)),
                                                    static_cast<unsigned char>( static_cast<double>(FNMR_COLOR_START.g())*2.0/3.0 + static_cast<double>(FNMR_COLOR_STOP.g()*1.0/3.0)),
                                                    static_cast<unsigned char>( static_cast<double>(FNMR_COLOR_START.b())*2.0/3.0 + static_cast<double>(FNMR_COLOR_STOP.b()*1.0/3.0)),
                                                },

                                                {

                                                    static_cast<unsigned char>( static_cast<double>(FNMR_COLOR_START.r())*1.0/3.0 + static_cast<double>(FNMR_COLOR_STOP.r()*2.0/3.0)),
                                                    static_cast<unsigned char>( static_cast<double>(FNMR_COLOR_START.g())*1.0/3.0 + static_cast<double>(FNMR_COLOR_STOP.g()*2.0/3.0)),
                                                    static_cast<unsigned char>( static_cast<double>(FNMR_COLOR_START.b())*1.0/3.0 + static_cast<double>(FNMR_COLOR_STOP.b()*2.0/3.0)),
                                                },
                                                FNMR_COLOR_STOP}};


const auto FMR_COLOR_START = tlp::Color::Red;
const auto FMR_COLOR_STOP  = tlp::Color::Blue;
static const std::array< tlp::Color, 4 > FMR_COLORS_LUT{{ FMR_COLOR_START,
                                                {
                                                    static_cast<unsigned char>( static_cast<double>(FMR_COLOR_START.r())*2.0/3.0 + static_cast<double>(FMR_COLOR_STOP.r()*1.0/3.0)),
                                                    static_cast<unsigned char>( static_cast<double>(FMR_COLOR_START.g())*2.0/3.0 + static_cast<double>(FMR_COLOR_STOP.g()*1.0/3.0)),
                                                    static_cast<unsigned char>( static_cast<double>(FMR_COLOR_START.b())*2.0/3.0 + static_cast<double>(FMR_COLOR_STOP.b()*1.0/3.0)),
                                                },

                                                {

                                                    static_cast<unsigned char>( static_cast<double>(FMR_COLOR_START.r())*1.0/3.0 + static_cast<double>(FMR_COLOR_STOP.r()*2.0/3.0)),
                                                    static_cast<unsigned char>( static_cast<double>(FMR_COLOR_START.g())*1.0/3.0 + static_cast<double>(FMR_COLOR_STOP.g()*2.0/3.0)),
                                                    static_cast<unsigned char>( static_cast<double>(FMR_COLOR_START.b())*1.0/3.0 + static_cast<double>(FMR_COLOR_STOP.b()*2.0/3.0)),
                                                },
                                                FMR_COLOR_STOP}};






namespace tlp {

class BiometricPowerNode : public Glyph
{
public:
#pragma clang diagnostic ignored "-Winconsistent-missing-override"
  GLYPHINFORMATION("BiometricPowerNode", "Romain Giot", "26/06/2017", "BiometricPowerNode representation", "1.0", 100)

  explicit BiometricPowerNode(const tlp::PluginContext *context);

  void getIncludeBoundingBox(BoundingBox &boundingBox, node) override;
  void getTextBoundingBox(BoundingBox &boundingBox,node) override;
  void draw(node n, float lod) override;
  Coord getAnchor(const Coord &vector) const override;

private:
  Color _rectColor;
  Color _borderColor;
  Color _fnmrColor, _fmrInColor, _fmrOutColor; // Color for the 3 additional boxes
  const float _borderWidth;
  GlPolyQuad _border;
  GlQuad _center;
  GlQuad _fnmr, _fmrIn, _fmrOut;
  BoundingBox _bb;
  BoundingBox _buttonsbb;
};

/*
 *
0------------------1 -0.5, 0.5      0.5, 0.5
|\                /|
| 4---12---13----5 |
| |fnmr|fmr1|fmr2| |
| 9---10---11----8 |-0.5 + _borderWith   0.5, 0.5-bo
| |  FREE        | |
| 7--------------6 |
|/                \|
3------------------2  -0.5,-0.5        0.5, -0.5
*/

PLUGIN(BiometricPowerNode)

BiometricPowerNode::BiometricPowerNode(const tlp::PluginContext *context)
: Glyph(context),
  _rectColor(205, 205, 205, 255),
  _borderColor(205, 205, 205, 255),
  _fnmrColor(tlp::Color::Red),
  _fmrInColor(tlp::Color::Green),
  _fmrOutColor(tlp::Color::Blue),
  _borderWidth(0.02f),
  _border(TulipBitmapDir + "halfCylinderTexture.png") {

  const float textheight = 0.05f;

  // TODO optimize the custruction with initilisation list
  Coord v[14];
  // outer border
  v[0].set(-0.5, 0.5, 0);
  v[1].set(0.5, 0.5, 0);
  v[2].set(0.5, -0.5, 0);
  v[3].set(-0.5, -0.5, 0);

  // inner border
  v[4].set(v[0][0] + _borderWidth, v[0][1] - _borderWidth, 0);
  v[5].set(v[1][0] - _borderWidth, v[1][1] - _borderWidth, 0);
  v[6].set(v[2][0] - _borderWidth, v[2][1] + _borderWidth, 0);
  v[7].set(v[3][0] + _borderWidth, v[3][1] + _borderWidth, 0);

  // content upper limit
  v[8].set(0.5 - _borderWidth, 0.5 - _borderWidth - textheight * 2.f, 0);
  v[9].set(-0.5 + _borderWidth, 0.5 - _borderWidth - textheight * 2.f, 0);

  // additional box limits
  v[10].set((v[8].x() - v[9].x()) / 3 + v[9].x(), v[8].y(), 0);
  v[11].set((v[8].x() - v[9].x()) * 2 / 3 + v[9].x(), v[8].y(), 0);

  v[12].set(v[10].x(), v[4].y(), 0);
  v[13].set(v[11].x(), v[4].y(), 0);


  // Inner bounding box
  // Initial size
  _bb.expand(v[6]);
  _bb.expand(v[9]);
  _bb.expand(v[8]);
  _bb.expand(v[7]);

  // Buttons bounding box
  _buttonsbb.expand(v[4]);
  _buttonsbb.expand(v[5]);
  _buttonsbb.expand(v[8]);
  _buttonsbb.expand(v[9]);

  // Create the outer border
  _border.addQuadEdge(v[0], v[4], _borderColor);
  _border.addQuadEdge(v[1], v[5], _borderColor);
  _border.addQuadEdge(v[2], v[6], _borderColor);
  _border.addQuadEdge(v[3], v[7], _borderColor);
  _border.addQuadEdge(v[0], v[4], _borderColor);


  // Place the 4 boxes
  _center.setPosition(0, v[9]);
  _center.setPosition(1, v[8]);
  _center.setPosition(2, v[6]);
  _center.setPosition(3, v[7]);

  _fnmr.setPosition(0, v[4]);
  _fnmr.setPosition(1, v[12]);
  _fnmr.setPosition(2, v[10]);
  _fnmr.setPosition(3, v[9]);


  _fmrIn.setPosition(0, v[12]);
  _fmrIn.setPosition(1, v[13]);
  _fmrIn.setPosition(2, v[11]);
  _fmrIn.setPosition(3, v[10]);

  _fmrOut.setPosition(0, v[13]);
  _fmrOut.setPosition(1, v[5]);
  _fmrOut.setPosition(2, v[8]);
  _fmrOut.setPosition(3, v[11]);
}
//=====================================================
void BiometricPowerNode::getIncludeBoundingBox(BoundingBox &boundingBox, node) {
  boundingBox = _bb;
}

void BiometricPowerNode::getTextBoundingBox(BoundingBox &boundingBox,node) {
  boundingBox = _buttonsbb /*_textbb*/;
  boundingBox.translate(tlp::Vec3f(_buttonsbb.width()/3,- _bb.height(), 0));
}


//=====================================================
void BiometricPowerNode::draw(node n, float lod) {
  ColorProperty *const color = glGraphInputData->getElementColor();
  ColorProperty *const colorBorder = glGraphInputData->getElementBorderColor();

  Graph *const graph = glGraphInputData->getGraph();
  // TODO optimize that to do it only one time
  unsigned int const clusterInFMR = graph->getProperty< IntegerProperty >(PROP_IN_FMR_CLUSTER)->getNodeValue(n);
  unsigned int const clusterOutFMR = graph->getProperty< IntegerProperty >(PROP_OUT_FMR_CLUSTER)->getNodeValue(n);
  unsigned int const clusterFNMR = graph->getProperty< IntegerProperty >(PROP_FNMR_CLUSTER)->getNodeValue(n);


  // Setup colors
  _center.setFillColor(color->getNodeValue(n));
  _center.draw(lod, NULL); // Do not draw center

  if (clusterInFMR < FMR_COLORS_LUT.size()) {
    _fmrIn.setColor(FMR_COLORS_LUT.at(clusterInFMR));
  } else {
    _fmrIn.setColor(Color::Black);
  }

  if (clusterOutFMR < FMR_COLORS_LUT.size()) {
    _fmrOut.setColor(FMR_COLORS_LUT.at(clusterOutFMR));
  } else {
    _fmrIn.setColor(Color::Black);
  }

  if (clusterFNMR < FNMR_COLORS_LUT.size()) {
    _fnmr.setColor(FNMR_COLORS_LUT.at(clusterFNMR));
  } else {
    _fmrIn.setColor(Color::Black);
  }
  _fmrOut.draw(lod, nullptr);
  _fmrIn.draw(lod, nullptr);
  _fnmr.draw(lod, nullptr);

  _border.setColor(colorBorder->getNodeValue(n));


  _border.draw(lod, NULL);
}

//=====================================================
Coord BiometricPowerNode::getAnchor(const Coord &vector) const {
  Coord v(vector);
  float x, y, z, fmax;
  v.get(x, y, z);
  v.setZ(0.0f);
  fmax = std::max(fabsf(x), fabsf(y));

  if (fmax > 0.0f)
    return v * (0.5f / fmax);
  else
    return v;
}

} // end of namespace tlp

#endif // BIOMETRIC_POWER_NODEGLYPH_H
