/**
 * Copyright University Bordeaux - CNRS UMR 5800 
 * contributor(s) : Romain Giot (2017)
 * 
 * Romain.Giot@u-bordeaux.fr
 * 
 * Biometric Power Graph (bpg) is a computer program whose purpose is to evaluate biometric authentication
 * systems by displaying with a graph-based representation after having transformed the data.
 * 
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use, 
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * http://www.cecill.info. 
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability. 
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or 
 * data to be ensured and,  more generally, to use and operate it in the 
 * same conditions as regards security. 
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */

// This file comes from a deep modification of the halfCircle plugin

#include <tulip/StringProperty.h>
#include <tulip/ColorProperty.h>
#include <tulip/DoubleProperty.h>
#include <tulip/Size.h>
#include <tulip/Coord.h>
#include <tulip/Glyph.h>
#include <tulip/EdgeExtremityGlyph.h>
#include <tulip/Graph.h>
#include <tulip/GlGraphRenderingParameters.h>
#include <tulip/GlGraphInputData.h>
#include <tulip/TulipViewSettings.h>
#include <tulip/GlCircle.h>


#include "bpg/GlHalfCircle.h"

using namespace std;
using namespace tlp;

namespace tlp {

static GlHalfCircle *halfCircles = nullptr;
static GlCircle *circle = nullptr;

static void drawHalfCircles(const Color &genuineColor, const Color &impostorColor, float lod) {
  halfCircles->setGenuineColor(genuineColor);
  halfCircles->setImpostorColor(impostorColor);
  halfCircles->draw(lod, nullptr);
}


static void drawCircle(const Color &mixedColor, float lod) {
  circle->setFillColor(mixedColor);

  circle->draw(lod, nullptr);
}




/** \addtogroup glyph */

/// A 2D glyph.
/**
 * This glyph draws a textured disc using the "viewTexture" node
 * property value. If this property has no value, the disc is then colored
 * using the "viewColor" node property value.
 */
class HalfCircle : public Glyph
{
public:
  GLYPHINFORMATION("2D - Half-Circles", "Romain Giot", "03/07/2017", "Textured Circle", "1.1", 90)
  HalfCircle(const tlp::PluginContext *context = NULL);
  virtual ~HalfCircle();
  virtual void getIncludeBoundingBox(BoundingBox &boundingBox, node);
  virtual void draw(node n, float lod);
};
PLUGIN(HalfCircle)
HalfCircle::HalfCircle(const tlp::PluginContext *context)
: Glyph(context) {
  if (!halfCircles) {
    halfCircles = new GlHalfCircle(Coord(0, 0, 0), 0.5, Color(0, 0, 0, 255), Color(0, 0, 0, 255), true, false, 0., 30);
    circle = new GlCircle(Coord(0, 0, 0), 0.5, Color(0, 0, 0, 255), Color(0, 0, 0, 255), true, false, 0., 30);

  }
}
HalfCircle::~HalfCircle() {
}
void HalfCircle::getIncludeBoundingBox(BoundingBox &boundingBox, node) {
  boundingBox[0] = Coord(-0.35f, -0.35f, 0);
  boundingBox[1] = Coord(0.35f, 0.35f, 0);
}

/**
 * Color correspondence:
 * - ElementBorderColor: genuineAttempts
 * - ElementLabelColor: impostorAttemps
 * - ElementColor: mixed attempts
 */
void HalfCircle::draw(node n, float lod) {
    const auto lodLimit = 20;

    if (lod > lodLimit) {
        halfCircles->setLightingMode(false);

        drawHalfCircles(
                    Glyph::glGraphInputData->getElementBorderColor()->getNodeValue(n),
                    Glyph::glGraphInputData->getElementLabelColor()->getNodeValue(n),
                    lod);
    }
    else {
        circle->setLightingMode(false);
         drawCircle(
                     Glyph::glGraphInputData->getElementColor()->getNodeValue(n),
                    lod);
       
    }
}


} // end of namespace tlp
