/**
 * Copyright University Bordeaux - CNRS UMR 5800 
 * contributor(s) : Romain Giot (2017)
 * 
 * Romain.Giot@u-bordeaux.fr
 * 
 * Biometric Power Graph (bpg) is a computer program whose purpose is to evaluate biometric authentication
 * systems by displaying with a graph-based representation after having transformed the data.
 * 
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use, 
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * http://www.cecill.info. 
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability. 
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or 
 * data to be ensured and,  more generally, to use and operate it in the 
 * same conditions as regards security. 
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */

/**
 *
 * This file is part of Tulip (www.tulip-software.org)
 *
 * Authors: David Auber and the Tulip development Team
 * from LaBRI, University of Bordeaux
 *
 * Tulip is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *
 * Tulip is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 */

#include "BiometricNeighborhoodHighlighterConfigWidget.h"
#include "ui_BiometricNeighborhoodHighlighterConfigWidget.h"

#include <tulip/TlpQtTools.h>

using namespace tlp;

BiometricNeighborhoodHighlighterConfigWidget::BiometricNeighborhoodHighlighterConfigWidget(QWidget *parent)
: QWidget(parent), _ui(new Ui::BiometricNeighborhoodHighlighterConfigWidgetData) {
  _ui->setupUi(this);
  connect(_ui->applyButton, SIGNAL(clicked()), this, SIGNAL(updateNeighborhoodGraph()));
}

BiometricNeighborhoodHighlighterConfigWidget::~BiometricNeighborhoodHighlighterConfigWidget() {
  delete _ui;
}

BiometricNodeNeighborhoodView::NeighborNodesType BiometricNeighborhoodHighlighterConfigWidget::getNeighborsType() const {
  if (_ui->outputEdgesRB->isChecked()) {
    return BiometricNodeNeighborhoodView::OUT_NEIGHBORS;
  } else if (_ui->inputEdgesRB->isChecked()) {
    return BiometricNodeNeighborhoodView::IN_NEIGHBORS;
  } else {
    return BiometricNodeNeighborhoodView::IN_OUT_NEIGHBORS;
  }
}

bool BiometricNeighborhoodHighlighterConfigWidget::computeReachableSubGraph() const {
  return _ui->computeReachableSGCB->isChecked();
}

void BiometricNeighborhoodHighlighterConfigWidget::setCurrentMaxDistanceForReachableNodes(unsigned int distance) {
  _ui->distanceLineEdit->setText(QString::number(distance));
}

bool BiometricNeighborhoodHighlighterConfigWidget::bringAndGoAnimation1() const {
  return _ui->bagAnimation1->isChecked();
}

int BiometricNeighborhoodHighlighterConfigWidget::numberOfNodesToBring() const {
  return _ui->nodesNumber->value();
}

std::string BiometricNeighborhoodHighlighterConfigWidget::propertyToUse() const {
  return tlp::QStringToTlpString(_ui->propertyName->currentText());
}

void BiometricNeighborhoodHighlighterConfigWidget::setNumberOfNodes(int nodesNb) {
  _ui->nodesNumber->setValue(nodesNb);
}

void BiometricNeighborhoodHighlighterConfigWidget::setPropertyToUse(std::string &propertyName) {
  _ui->propertyName->addItem(tlpStringToQString(propertyName));
  _ui->propertyName->setCurrentIndex(_ui->propertyName->findText(tlpStringToQString(propertyName)));
}

bool BiometricNeighborhoodHighlighterConfigWidget::isdisplayEdgesCBChecked() const {
  return _ui->displayEdgesCB->isChecked();
}
