/**
 * Copyright University Bordeaux - CNRS UMR 5800 
 * contributor(s) : Romain Giot (2017)
 * 
 * Romain.Giot@u-bordeaux.fr
 * 
 * Biometric Power Graph (bpg) is a computer program whose purpose is to evaluate biometric authentication
 * systems by displaying with a graph-based representation after having transformed the data.
 * 
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use, 
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * http://www.cecill.info. 
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability. 
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or 
 * data to be ensured and,  more generally, to use and operate it in the 
 * same conditions as regards security. 
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */

#ifndef BIOMETRICDATABASEFORM_H
#define BIOMETRICDATABASEFORM_H

#include <QGroupBox>
#include <QAbstractItemModel>

namespace Ui {
class BiometricDatabaseForm;
}

class BiometricDatabaseForm : public QGroupBox
{
  Q_OBJECT

public:
  explicit BiometricDatabaseForm(QWidget *parent = 0);
  ~BiometricDatabaseForm();

  void setModel(QAbstractItemModel *model);
  void pgIsDisplayed() {
      _pgDisplayed = true;
  }


  void informIsGalleryFiltered(){
      is_gallery_filtered = true;
  }
  void informIsProbeFiltered(){
      is_probe_filtered = true;
  }
  void informIsSampleFiltered(){
      is_sample_filtered = true;
  }

private slots:
  void showContextMenu(const QPoint &);
  void applySearch();

signals:
  void filterGallery(const QString gallery_id);
  void filterProbe(const QString probe_id);
  void filterSample(const QString probe_id, const QString sample_id);
  void clearFilters();

  void requestFocusSample(int indiv, int sample);
  void requestFocusIndividual(int indiv);

private:
  Ui::BiometricDatabaseForm *ui;
  QAbstractItemModel *_model;


  // These 3 flags change while the use of the filtering
   bool is_gallery_filtered = false;
   bool is_probe_filtered = false;
   bool is_sample_filtered = false;

   bool _pgDisplayed = false;
};

#endif // BIOMETRICDATABASEFORM_H
