/**
 * Copyright University Bordeaux - CNRS UMR 5800 
 * contributor(s) : Romain Giot (2017)
 * 
 * Romain.Giot@u-bordeaux.fr
 * 
 * Biometric Power Graph (bpg) is a computer program whose purpose is to evaluate biometric authentication
 * systems by displaying with a graph-based representation after having transformed the data.
 * 
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use, 
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * http://www.cecill.info. 
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability. 
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or 
 * data to be ensured and,  more generally, to use and operate it in the 
 * same conditions as regards security. 
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */

#ifndef POWERGRAPHFORM_H
#define POWERGRAPHFORM_H

#include <memory>

#include <QGraphicsView>
#include <QWidget>

#include <tulip/MouseInteractors.h>
#include <tulip/MouseSelectionEditor.h>
#include <tulip/View.h>
#include <tulip/GlGraphRenderingParameters.h>
#include "bpg/interactors/PowerGraphInteractor.h"


#include <QSettings>


class PowerGraphInteractor;

namespace tlp {
class Graph;
class View;
class GlMainWidget;
class GlLayer;
class BiometricNeighborhoodHighlighterInteractor ;
}

namespace Ui {
class PowerGraphForm;
}

class PowerGraphForm : public QWidget
{
  Q_OBJECT

public:
  explicit PowerGraphForm(QWidget *parent, tlp::Graph *dummyGraph);
  ~PowerGraphForm();

  PowerGraphInteractor *pgInteractor() { return &_pgInteractor; }
  tlp::Graph * getLayoutToCloneGraph() { return _layoutGraph;}

signals:
  void infoToggled(bool);
  void individualOrSampleSelected(tlp::node);
  void layoutImported();
  void clickOnExportLayout();
  void selectIndividualOrSample(tlp::node);


private slots:
  void clickOnImportLayout();
  void moveLeft();
  void moveRight();
  void moveUp();
  void moveDown();
  void zoomIn();
  void zoomOut();

public slots:
  void setPowerGraph(tlp::Graph *graph, double threshold = -1, double fnmr = -1, double fmr = -1);
  void showEdges(bool);
  void showContextMenu(const QPoint&);
  void takeScreenshot();
  void centerView();

  void focusIndividual(int individual);
  void focusSample(int individual, int sample);



private:
  Ui::PowerGraphForm *ui;
  tlp::View *_pgraphView;
  std::unique_ptr< tlp::MouseNKeysNavigator > _pgraphNavigator;
  tlp::GlGraphRenderingParameters _renderingParameters;
  PowerGraphInteractor _pgInteractor;

  tlp::Graph * _layoutGraph;
  tlp::Graph * graph;
  QSettings _settings;

  tlp::BiometricNeighborhoodHighlighterInteractor *neighInteractor;
};

#endif // POWERGRAPHFORM_H
