/**
 * Copyright University Bordeaux - CNRS UMR 5800 
 * contributor(s) : Romain Giot (2017)
 * 
 * Romain.Giot@u-bordeaux.fr
 * 
 * Biometric Power Graph (bpg) is a computer program whose purpose is to evaluate biometric authentication
 * systems by displaying with a graph-based representation after having transformed the data.
 * 
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use, 
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * http://www.cecill.info. 
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability. 
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or 
 * data to be ensured and,  more generally, to use and operate it in the 
 * same conditions as regards security. 
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */

#ifndef ROCFORM_H
#define ROCFORM_H


#include <tulip/Node.h>
#include <tulip/GlMainWidget.h>
#include "bpg/interactors/ROCInteractor.h"
#include <memory>

namespace tlp {
class GlMainWidget;
class Graph;
}

namespace Ui {
class ROCForm;
}

class ROCForm : public QWidget
{
  Q_OBJECT

public:
  explicit ROCForm(QWidget *parent = 0);
  ~ROCForm();


  ROCInteractor &getROCInteractor() {
    return *(_rocNavigator.get());
  }

  /**
     * Change the ROC graph to display
     */
  void setGraph(tlp::Graph *);


private:
  /**
     * At the moment, most of task are delegated to the main window / it will have to change later
     * @brief getGlMainWidget
     * @return
     */
  tlp::GlMainWidget *getGlMainWidget();


signals:
  void rocNodeSelected(tlp::node);
  void manualThresholdValueSelected(double threshold);

public slots:
  void eerSelected();
private slots:
  void manualThresholdButtonSelected();



private:
  Ui::ROCForm *ui;
  tlp::Graph *_root;
  tlp::GlLayer *_rocLayer;
  std::unique_ptr< ROCInteractor > _rocNavigator;
};

#endif // ROCFORM_H
