/**
 * Copyright University Bordeaux - CNRS UMR 5800 
 * contributor(s) : Romain Giot (2017)
 * 
 * Romain.Giot@u-bordeaux.fr
 * 
 * Biometric Power Graph (bpg) is a computer program whose purpose is to evaluate biometric authentication
 * systems by displaying with a graph-based representation after having transformed the data.
 * 
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use, 
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * http://www.cecill.info. 
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability. 
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or 
 * data to be ensured and,  more generally, to use and operate it in the 
 * same conditions as regards security. 
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */

// Original code from http://wiki.qt.io/PushButton_Based_On_Action

#include "bpg/ActionButton.h"

ActionButton::ActionButton(QWidget *parent)
: QPushButton(parent),
  actionOwner(nullptr) {}

void ActionButton::setAction(QAction *action) {
  // if I've got already an action associated to the button
  // remove all connections
  if (actionOwner && actionOwner != action) {
    disconnect(actionOwner, SIGNAL(changed()),
               this, SLOT(updateButtonStatusFromAction()));
    disconnect(this, SIGNAL(clicked()),
               actionOwner, SLOT(trigger()));
  }

  // store the action
  actionOwner = action;

  // configure the button
  updateButtonStatusFromAction();

  // connect the action and the button
  // so that when the action is changed the
  // button is changed too!
  connect(action, SIGNAL(changed()),
          this, SLOT(updateButtonStatusFromAction()));

  // connect the button to the slot that forwards the
  // signal to the action
  connect(this, SIGNAL(clicked()),
          actionOwner, SLOT(trigger()));
}

void ActionButton::updateButtonStatusFromAction() {
  if (!actionOwner)
    return;
  setText(actionOwner->text());
  setStatusTip(actionOwner->statusTip());
  setToolTip(actionOwner->toolTip());
  setIcon(actionOwner->icon());
  setEnabled(actionOwner->isEnabled());
  setCheckable(actionOwner->isCheckable());
  setChecked(actionOwner->isChecked());
  setShortcut(actionOwner->shortcut());
}
