/**
 * Copyright University Bordeaux - CNRS UMR 5800 
 * contributor(s) : Romain Giot (2017)
 * 
 * Romain.Giot@u-bordeaux.fr
 * 
 * Biometric Power Graph (bpg) is a computer program whose purpose is to evaluate biometric authentication
 * systems by displaying with a graph-based representation after having transformed the data.
 * 
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use, 
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * http://www.cecill.info. 
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability. 
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or 
 * data to be ensured and,  more generally, to use and operate it in the 
 * same conditions as regards security. 
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */

#include "bpg/ClusterColorLegend.h"
#include "ui_ClusterColorLegend.h"
#include <array>


#include <tulip/Color.h>

// XXX Attention, this is a copy past of BiometricPowerNode
const auto FNMR_COLOR_START = tlp::Color::Gray;
const auto FNMR_COLOR_STOP  = tlp::Color::Green;
static const std::array< tlp::Color, 4 > FNMR_COLORS_LUT{{ FNMR_COLOR_START,
                                                {
                                                    static_cast<unsigned char>( static_cast<double>(FNMR_COLOR_START.r())*2.0/3.0 + static_cast<double>(FNMR_COLOR_STOP.r()*1.0/3.0)),
                                                    static_cast<unsigned char>( static_cast<double>(FNMR_COLOR_START.g())*2.0/3.0 + static_cast<double>(FNMR_COLOR_STOP.g()*1.0/3.0)),
                                                    static_cast<unsigned char>( static_cast<double>(FNMR_COLOR_START.b())*2.0/3.0 + static_cast<double>(FNMR_COLOR_STOP.b()*1.0/3.0)),
                                                },

                                                {

                                                    static_cast<unsigned char>( static_cast<double>(FNMR_COLOR_START.r())*1.0/3.0 + static_cast<double>(FNMR_COLOR_STOP.r()*2.0/3.0)),
                                                    static_cast<unsigned char>( static_cast<double>(FNMR_COLOR_START.g())*1.0/3.0 + static_cast<double>(FNMR_COLOR_STOP.g()*2.0/3.0)),
                                                    static_cast<unsigned char>( static_cast<double>(FNMR_COLOR_START.b())*1.0/3.0 + static_cast<double>(FNMR_COLOR_STOP.b()*2.0/3.0)),
                                                },
                                                FNMR_COLOR_STOP}};


const auto FMR_COLOR_START = tlp::Color::Red;
const auto FMR_COLOR_STOP  = tlp::Color::Blue;
static const std::array< tlp::Color, 4 > FMR_COLORS_LUT{{ FMR_COLOR_START,
                                                {
                                                    static_cast<unsigned char>( static_cast<double>(FMR_COLOR_START.r())*2.0/3.0 + static_cast<double>(FMR_COLOR_STOP.r()*1.0/3.0)),
                                                    static_cast<unsigned char>( static_cast<double>(FMR_COLOR_START.g())*2.0/3.0 + static_cast<double>(FMR_COLOR_STOP.g()*1.0/3.0)),
                                                    static_cast<unsigned char>( static_cast<double>(FMR_COLOR_START.b())*2.0/3.0 + static_cast<double>(FMR_COLOR_STOP.b()*1.0/3.0)),
                                                },

                                                {

                                                    static_cast<unsigned char>( static_cast<double>(FMR_COLOR_START.r())*1.0/3.0 + static_cast<double>(FMR_COLOR_STOP.r()*2.0/3.0)),
                                                    static_cast<unsigned char>( static_cast<double>(FMR_COLOR_START.g())*1.0/3.0 + static_cast<double>(FMR_COLOR_STOP.g()*2.0/3.0)),
                                                    static_cast<unsigned char>( static_cast<double>(FMR_COLOR_START.b())*1.0/3.0 + static_cast<double>(FMR_COLOR_STOP.b()*2.0/3.0)),
                                                },
                                                FMR_COLOR_STOP}};





ClusterColorLegend::ClusterColorLegend(QWidget *parent)
: QDialog(parent),
  ui(new Ui::ClusterColorLegend) {
  ui->setupUi(this);


  constexpr int max_value = 4;


  const std::array< QString, 4 > rankedText = {{"very high", "high", "low", "very low"}};
  ui->tableWidget->setRowCount(max_value);


  auto tlp2qt = [](const tlp::Color &c) {
    return QColor(c.r(), c.g(), c.b());
  };

  auto update_row = [&](unsigned int row) {
      
        auto itemi = new QTableWidgetItem(rankedText[row]);
        auto itemj = new QTableWidgetItem("");
        auto itemk = new QTableWidgetItem("");
        auto iteml = new QTableWidgetItem("");

        itemj->setBackgroundColor(tlp2qt(FNMR_COLORS_LUT[row]));
        itemk->setBackgroundColor(tlp2qt(FMR_COLORS_LUT[row]));
        iteml->setBackgroundColor(tlp2qt(FMR_COLORS_LUT[row]));


        ui->tableWidget->setItem(row, 0, itemi);
        ui->tableWidget->setItem(row, 1, itemj);
        ui->tableWidget->setItem(row, 2, itemk);
        ui->tableWidget->setItem(row, 3, iteml);

  };

  for (unsigned int i=0; i<max_value; ++i) {
    update_row(i);
  }

}

ClusterColorLegend::~ClusterColorLegend() {
  delete ui;
}
