#include "bpg/GenuineScoreEvolution.h"
#include "ui_GenuineScoreEvolution.h"

#include <numeric>
#include <vector>

GenuineScoreEvolution::GenuineScoreEvolution(QWidget *parent)
: QWidget(parent),
  ui(new Ui::GenuineScoreEvolution) {
  ui->setupUi(this);

  ui->genuineScorePlot->setAxisTitle(QwtPlot::xBottom, "Query order");
  ui->genuineScorePlot->setAxisTitle(QwtPlot::yLeft, "Genuine score");

  ui->genuineScorePlot->axisAutoScale(QwtPlot::xBottom);
  ui->genuineScorePlot->axisAutoScale(QwtPlot::yLeft);

  _curveRejected = _curveAccepted = nullptr;
  
  

}

GenuineScoreEvolution::~GenuineScoreEvolution() {
  delete ui;
}


void GenuineScoreEvolution::setGenuineScores(int userid, tlp::node usernode, double /*thr*/, const std::vector< double > &scores, const std::vector< bool > &acceptance) {

  this->userid = userid;
  this->usernode = usernode;

  this->ui->labelIndividual->setText(QString::number(userid));

  assert(usernode.isValid());
  assert(scores.size() == acceptance.size());

  if (_curveAccepted != nullptr) {
    _curveAccepted->detach();
    delete _curveAccepted;
    _curveAccepted = nullptr;

    _curveRejected->detach();
    delete _curveRejected;
    _curveRejected = nullptr;
  }



  _xAccepted.clear();
  _yAccepted.clear();
  _xRejected.clear();
  _yRejected.clear();


  QwtSymbol *symbolAccepted = new QwtSymbol();
  QwtSymbol *symbolRejected = new QwtSymbol();

  const QColor accepted = QColor(0, 255, 0);
  const QColor rejected = QColor(128, 128, 128);
  symbolAccepted->setStyle(QwtSymbol::Ellipse);
  symbolAccepted->setSize(QSize(7, 7));
  symbolAccepted->setBrush(accepted);


   symbolRejected->setStyle(QwtSymbol::Ellipse);
   symbolRejected->setSize(QSize(7, 7));
   symbolRejected->setBrush(rejected);


  for (unsigned int i = 0; i < scores.size(); ++i) {
    if (acceptance[i]) {
			_yAccepted.push_back(scores[i]);
			_xAccepted.push_back(i + 1);
    } else {
      _yRejected.push_back(scores[i]);
      _xRejected.push_back(i + 1);
    }
  }


  _curveAccepted = new QwtPlotCurve("Accepted");
  _curveRejected = new QwtPlotCurve("Rejected");
  

  _curveAccepted->setRawSamples(_xAccepted.data(), _yAccepted.data(), _xAccepted.size());
  _curveRejected->setRawSamples(_xRejected.data(), _yRejected.data(), _xRejected.size());

  _curveAccepted->setSymbol(symbolAccepted);
  _curveRejected->setSymbol(symbolRejected);




  
  _curveAccepted->setStyle(QwtPlotCurve::NoCurve);
  _curveRejected->setStyle(QwtPlotCurve::NoCurve);

  



  _curveAccepted->attach(ui->genuineScorePlot);
  _curveRejected->attach(ui->genuineScorePlot);

  ui->genuineScorePlot->replot();
}
