/**
 * Copyright University Bordeaux - CNRS UMR 5800 
 * contributor(s) : Romain Giot (2017)
 * 
 * Romain.Giot@u-bordeaux.fr
 * 
 * Biometric Power Graph (bpg) is a computer program whose purpose is to evaluate biometric authentication
 * systems by displaying with a graph-based representation after having transformed the data.
 * 
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use, 
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * http://www.cecill.info. 
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability. 
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or 
 * data to be ensured and,  more generally, to use and operate it in the 
 * same conditions as regards security. 
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */

#include "bpg/ImportDialog.h"
#include "ui_ImportDialog.h"

#include <QFileDialog>
#include <QFileInfo>
#include <QStatusBar>

ImportDialog::ImportDialog(QWidget *parent)
: QDialog(parent),
  ui(new Ui::ImportDialog) {
  ui->setupUi(this);


  auto bar = new QStatusBar(this);
  ui->statusBarPosition->addWidget(bar);


  connect(this->ui->fileButton, SIGNAL(pressed()),
          this, SLOT(fileButtonSelection()));

  connect(this->ui->thumbnailButton, SIGNAL(pressed()),
          this, SLOT(thumbnailButtonSelection()));

  connect(this->ui->fileEdit, SIGNAL(textChanged(QString)),
          this, SLOT(checkFileValidity(QString)));
  connect(this, SIGNAL(accepted()),
          this, SLOT(isAccepted()));

  const QString fname = _settings.value("import/fname", "").toString();
  const bool isScore = _settings.value("import/isScore", true).toBool();


  this->ui->fileEdit->setText(fname);
  this->ui->checkBoxScore->setChecked(isScore);

  this->ui->thumbnailEdit->setText(_settings.value("import/thumbnailFname").toString());
  this->ui->checkBoxThumbnail->setChecked(_settings.value("import/thumbnailUse").toBool());
}

ImportDialog::~ImportDialog() {
  delete ui;
}


void ImportDialog::fileButtonSelection() {

  const QString fileName = QFileDialog::getOpenFileName(
    this,
    tr("Choose CSV score file to import"),
    QDir::homePath(),
    tr("*.csv"),
    0,
    QFileDialog::DontUseNativeDialog);
  this->ui->fileEdit->setText(fileName);
}

void ImportDialog::thumbnailButtonSelection() {

  const QString fileName = QFileDialog::getOpenFileName(
    this,
    tr("Choose CSV thumbnail file to import"),
    QDir::homePath(),
    tr("*.csv"),
    0,
    QFileDialog::DontUseNativeDialog);
  this->ui->thumbnailEdit->setText(fileName);
}



QString ImportDialog::filename() {
  return this->ui->fileEdit->text();
}

QString ImportDialog::thumbnailFname() {
    if (this->ui->checkBoxThumbnail->isChecked()) {
        return this->ui->thumbnailEdit->text();
    }
    else {
        return "";
    }
}


bool ImportDialog::isScore() {
  return this->ui->checkBoxScore->isChecked();
}


QPushButton *ImportDialog::getOpenButton() {
  return this->ui->buttonBox->button(QDialogButtonBox::Open);
}


void ImportDialog::checkFileValidity(QString fname) {
  QFileInfo info(fname);
  if (!info.exists() or !info.isFile()) {
    getOpenButton()->setEnabled(false);
  } else {
    getOpenButton()->setEnabled(true);
  }
}


void ImportDialog::isAccepted() {
  _settings.setValue("import/fname", this->ui->fileEdit->text());
  _settings.setValue("import/isScore", this->ui->checkBoxScore->isChecked());
  _settings.setValue("import/thumbnailFname", this->ui->thumbnailEdit->text());
  _settings.setValue("import/thumbnailUse", this->ui->checkBoxThumbnail->isChecked());
}
