#include "bpg/ImpostorScoreEvolution.h"
#include "ui_ImpostorScoreEvolution.h"

#include <numeric>
#include <vector>

ImpostorScoreEvolution::ImpostorScoreEvolution(QWidget *parent)
: QWidget(parent),
  ui(new Ui::ImpostorScoreEvolution) {
  ui->setupUi(this);

  ui->impostorScorePlot->setAxisTitle(QwtPlot::xBottom, "Query order");
  ui->impostorScorePlot->setAxisTitle(QwtPlot::yLeft, "Impostor score");

  ui->impostorScorePlot->axisAutoScale(QwtPlot::xBottom);
  ui->impostorScorePlot->axisAutoScale(QwtPlot::yLeft);

  _curveRejected = _curveAccepted = nullptr;
}

ImpostorScoreEvolution::~ImpostorScoreEvolution() {
  delete ui;
}


/**
 * @todo adapt the display to different cases 1. sample compared to everybody 2. sample compared to one guy one or several times
 */
void ImpostorScoreEvolution::setImpostorScores(    int userid, 
    int sampleid,
    tlp::node usernode, 
    tlp::node /*samplenode*/,
    double /*thr*/, 
    const std::vector< double > &scores	,
    const std::vector< bool > &	acceptance
    ) {

  this->userid = userid;
  this->usernode = usernode;

  this->ui->labelIndividual->setText(QString::number(userid));
  this->ui->labelSample->setText(QString::number(sampleid));

  assert(usernode.isValid());
  assert(scores.size() == acceptance.size());

  if (_curveAccepted != nullptr) {
    _curveAccepted->detach();
    delete _curveAccepted;
    _curveAccepted = nullptr;

    _curveRejected->detach();
    delete _curveRejected;
    _curveRejected = nullptr;
  }

  const QColor accepted = QColor(255, 0, 0);
  const QColor rejected = QColor(0, 0, 255);


  _xAccepted.clear();
  _yAccepted.clear();
  _xRejected.clear();
  _yRejected.clear();

  for (unsigned int i = 0; i < scores.size(); ++i) {
    if (acceptance[i]) {
      _yAccepted.push_back(scores[i]);
      _xAccepted.push_back(i + 1);
    } else {

      _yRejected.push_back(scores[i]);
      _xRejected.push_back(i + 1);
    }
  }

  _curveAccepted = new QwtPlotCurve("Accepted");
  _curveAccepted->setRawSamples(_xAccepted.data(), _yAccepted.data(), _xAccepted.size());
  _curveAccepted->setStyle(QwtPlotCurve::NoCurve);

  _curveRejected = new QwtPlotCurve("Rejected");
  _curveRejected->setRawSamples(_xRejected.data(), _yRejected.data(), _xRejected.size());
  _curveRejected->setStyle(QwtPlotCurve::NoCurve);

  QwtSymbol *symbolAccepted = new QwtSymbol(); // XXX managed by qwt
  symbolAccepted->setStyle(QwtSymbol::Ellipse);
  symbolAccepted->setSize(QSize(7, 7));
  symbolAccepted->setBrush(accepted);

  QwtSymbol *symbolRejected = new QwtSymbol(); // XXX Managed by qwt
  symbolRejected->setStyle(QwtSymbol::Ellipse);
  symbolRejected->setSize(QSize(7, 7));
  symbolRejected->setBrush(rejected);

  _curveAccepted->setSymbol(symbolAccepted);
  _curveRejected->setSymbol(symbolRejected);

  _curveAccepted->attach(ui->impostorScorePlot);
  _curveRejected->attach(ui->impostorScorePlot);

  ui->impostorScorePlot->replot();

}
