#include "bpg/ROCWindow.h"
#include "ui_ROCWindow.h"

#include <qwt_plot.h>
#include <qwt_plot_grid.h>
#include <qwt_symbol.h>
#include <qwt_legend.h>
#include <qwt_scale_engine.h>

#include "tulip/Graph.h"

#include "bpg/common.h"

#include <QMenu>
#include <QPoint>
#include <QFileDialog>
#include <qwt_plot_renderer.h>

ROCWindow::ROCWindow(QWidget *parent, PowerGraphManager &pg)
: QMainWindow(parent),
  ui(new Ui::ROCWindow),
  _pgManager(pg),
  _roc("c"),
  _det("c") {
  ui->setupUi(this);


  // Get the scores
  tlp::Graph *const g = _pgManager.getROCGraph();
  auto const FNMR = g->getProperty< tlp::DoubleProperty >(PROP_FNMR);
  auto const FMR = g->getProperty< tlp::DoubleProperty >(PROP_FMR);
  for (tlp::node n : g->nodes()) {
    _TMR.push_back(1.0 - FNMR->getNodeValue(n));
    _FNMR.push_back(FNMR->getNodeValue(n));
    _FMR.push_back(FMR->getNodeValue(n));
  }

  // Create the plot

  _roc.setRawSamples(_FMR.data(), _TMR.data(), _FMR.size());
  _det.setRawSamples(_FMR.data(), _FNMR.data(), _FMR.size());

  connect(ui->rocRadioButton, SIGNAL(toggled(bool)), this, SLOT(showRoc(bool)));
  connect(ui->detRadioButton, SIGNAL(toggled(bool)), this, SLOT(showDet(bool)));

  connect(
              ui->widgetPlot, SIGNAL(customContextMenuRequested(const QPoint&)),
      this, SLOT(myContextMenu(const QPoint&)));


  QPen pen(Qt::black, 4);
  _roc.setPen(pen);
  _det.setPen(pen);

  ui->widgetPlot->setStyleSheet("background-color: white");
  showRoc(true);
}

void ROCWindow::myContextMenu(const QPoint& p) {

    QMenu myMenu(ui->widgetPlot);
    QAction* itemPDF = myMenu.addAction("Export to PDF");
    myMenu.move(ui->widgetPlot->mapToGlobal(p));

    QAction* selectedItem = myMenu.exec();
    if (selectedItem == itemPDF) {
        QString fname = QFileDialog::getSaveFileName(this,
                                                     "Select file to export ROC curve",
                                                     "",
                                                     "*.pdf");
         if (!fname.isEmpty()) {
            QwtPlotRenderer renderer;
            renderer.renderDocument( ui->widgetPlot, fname, QSizeF( 300, 200 ), 85 );
         }

    }
}

void ROCWindow::showRoc(bool show) {

  if (show) {
    std::cout << "ROC" << std::endl;
    _roc.attach(ui->widgetPlot);

    ui->widgetPlot->setAxisTitle(QwtPlot::xBottom, "False Match Rate");
    ui->widgetPlot->setAxisTitle(QwtPlot::yLeft, "True Match Rate");

    ui->widgetPlot->setAxisScaleEngine(QwtPlot::yLeft, new QwtLinearScaleEngine());
    ui->widgetPlot->setAxisScaleEngine(QwtPlot::xBottom, new QwtLinearScaleEngine());

    ui->widgetPlot->replot();
  } else {
    _roc.detach();
  }

}


void ROCWindow::showDet(bool show) {
  if (show) {
    std::cout << "DET" << std::endl;
    _det.attach(ui->widgetPlot);

    ui->widgetPlot->setAxisTitle(QwtPlot::xBottom, "False Match Rate");
    ui->widgetPlot->setAxisTitle(QwtPlot::yLeft, "False Non Match Rate");


    ui->widgetPlot->setAxisScaleEngine(QwtPlot::yLeft, new QwtLogScaleEngine());
    ui->widgetPlot->setAxisScaleEngine(QwtPlot::xBottom, new QwtLogScaleEngine());


    ui->widgetPlot->replot();
  } else {
    _det.detach();
  }
}

ROCWindow::~ROCWindow() {
  delete ui;
}
