/**
 * Copyright University Bordeaux - CNRS UMR 5800 
 * contributor(s) : Romain Giot (2017)
 * 
 * Romain.Giot@u-bordeaux.fr
 * 
 * Biometric Power Graph (bpg) is a computer program whose purpose is to evaluate biometric authentication
 * systems by displaying with a graph-based representation after having transformed the data.
 * 
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use, 
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * http://www.cecill.info. 
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability. 
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or 
 * data to be ensured and,  more generally, to use and operate it in the 
 * same conditions as regards security. 
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */

#include "bpg/SampleColorLegend.h"
#include "ui_SampleColorLegend.h"

#include <array>


SampleColorLegend::SampleColorLegend(QWidget *parent)
: QDialog(parent),
  ui(new Ui::SampleColorLegend) {
  ui->setupUi(this);


  constexpr int nb_rows = 8;
  ui->tableWidget->setRowCount(nb_rows);

  auto update_row = [&](unsigned int row, QString position, QString significance, QColor color) {
      
        auto itemi = new QTableWidgetItem(position);
        auto itemj = new QTableWidgetItem(significance);
        auto itemk = new QTableWidgetItem("");

        itemk->setBackgroundColor(color);


        ui->tableWidget->setItem(row, 0, itemi);
        ui->tableWidget->setItem(row, 1, itemj);
        ui->tableWidget->setItem(row, 2, itemk);

  };

  // TODO average colors in another space than the RGB one
  auto mix = [](const QColor &a, const QColor &b) {
    return QColor{
        (a.red() + b.red())/2,
        (a.green() + b.green())/2,
        (a.blue() + b.blue())/2
    };
  };


  // XXX Should be retreived from BiometricDatabaseModel
  static const QColor trueMatch{0, 255, 0};
  static const QColor falseNonMatch{128, 128, 128};
  static const QColor trueNonMatch{0, 0, 255};
  static const QColor falseMatch{255, 0, 0};


  // Correspondances are in DrawBiometricPowerGraph::computeSamplesColors
  update_row(0, "Left or full", "True Match", trueMatch); // Green
  update_row(1, "Left or full", "False Non Match", falseNonMatch);     // Gray

  update_row(2, "Right or full", "Always True Non Match", trueNonMatch); // Blue
  update_row(3, "Right or full", "At least one  False Match", falseMatch); // Red
  
  update_row(4, "Full (small)", "True Match & Always True Non Match", mix(trueMatch, trueNonMatch)); 
  update_row(5, "Full (small)", "True Match & At least one False Match", mix(trueMatch, falseMatch)); 
  update_row(6, "Full (small)", "False Non Match & Always True Non Match", mix(falseNonMatch, trueNonMatch)); 
  update_row(7, "Full (small)", "False Non Match & At least one False Match", mix(falseNonMatch, falseMatch)); 

}

SampleColorLegend::~SampleColorLegend() {
  delete ui;
}
