#include "ui/include/bpg/ThresholdToErrorsWindow.h"
#include "ui_ThresholdToErrorsWindow.h"

#include "bpg/common.h"
#include "bpg/powergraphmanager.h"
#include <QMenu>
#include <QPoint>
#include <QFileDialog>
#include <qwt_plot_renderer.h>
#include <qwt_legend.h>

ThresholdToErrorsWindow::ThresholdToErrorsWindow(QWidget *parent, PowerGraphManager &pg) :
    QMainWindow(parent),
    ui(new Ui::ThresholdToErrorsWindow)
{
    ui->setupUi(this);

  ui->widgetPlot->setContextMenuPolicy(Qt::CustomContextMenu);
  connect(
              ui->widgetPlot, SIGNAL(customContextMenuRequested(const QPoint&)),
      this, SLOT(myContextMenu(const QPoint&)));

    buildGraph(pg);
}


void ThresholdToErrorsWindow::buildGraph(PowerGraphManager &pg) {
    // get the data
  tlp::Graph *const g = pg.getROCGraph();
  auto const FNMR = g->getProperty< tlp::DoubleProperty >(PROP_FNMR);
  auto const FMR = g->getProperty< tlp::DoubleProperty >(PROP_FMR);
  auto const THR = g->getProperty< tlp::DoubleProperty >(PROP_THR);

  auto const nb_nodes = g->numberOfNodes();
  _fnmr.reserve(nb_nodes);
  _fmr.reserve(nb_nodes);
  _thresholds.reserve(nb_nodes);

  for (const tlp::node &n : g->nodes()) {
    _fnmr.push_back(FNMR->getNodeValue(n));
    _fmr.push_back(FMR->getNodeValue(n));
    _thresholds.push_back(THR->getNodeValue(n));
  }


  // Build the graph
  _fmrCurve.setRawSamples(_thresholds.data(), _fmr.data(), nb_nodes);
  _fnmrCurve.setRawSamples(_thresholds.data(), _fnmr.data(), nb_nodes);

  QPen fmrPen(QColor(241,163,64), 4);
  QPen fnmrPen(QColor(153,142,195), 4);

  _fmrCurve.setPen(fmrPen);
  _fnmrCurve.setPen(fnmrPen);

  ui->widgetPlot->setStyleSheet("background-color: white");
    ui->widgetPlot->setAxisTitle(QwtPlot::xBottom, "Decision threshold");
    ui->widgetPlot->setAxisTitle(QwtPlot::yLeft, "Error rate");

    _fmrCurve.setTitle("False Match Rate");
    _fnmrCurve.setTitle("False Non Match Rate");

    _fmrCurve.attach(ui->widgetPlot);
    _fnmrCurve.attach(ui->widgetPlot);

    ui->widgetPlot->insertLegend(new QwtLegend(ui->widgetPlot), QwtPlot::TopLegend);


    //ui->widgetPlot->replot();
}

ThresholdToErrorsWindow::~ThresholdToErrorsWindow()
{
    delete ui;
}

void ThresholdToErrorsWindow::myContextMenu(const QPoint& p) {

    QMenu myMenu(ui->widgetPlot);
    QAction* itemPDF = myMenu.addAction("Export to PDF");
    myMenu.move(ui->widgetPlot->mapToGlobal(p));

    QAction* selectedItem = myMenu.exec();
    if (selectedItem == itemPDF) {
        QString fname = QFileDialog::getSaveFileName(this,
                                                     "Select file to export FMR & FNMR curve",
                                                     "",
                                                     "*.pdf");
         if (!fname.isEmpty()) {
            QwtPlotRenderer renderer;
            renderer.renderDocument( ui->widgetPlot, fname, QSizeF( 300, 200 ), 85 );
         }

    }
}


