#include "ui_ZooPlotWindow.h"
#include <qwt_symbol.h>
#include "bpg/ZooPlotWindow.h"

#include <algorithm>
#include <QMenu>
#include <QPoint>
#include <QFileDialog>
#include <qwt_plot_renderer.h>


std::pair<double, double> getQuartiles(std::vector<double> quantile) {

    auto const Q1 = quantile.size() / 4;
    auto const Q2 = quantile.size() / 2;
    auto const Q3 = Q1 + Q2;

    std::sort(quantile.begin(), quantile.end());

    return {quantile[Q1], quantile[Q3]};

}


ZooPlotWindow::ZooPlotWindow(QWidget *parent,  PowerGraphManager &pg)
    : QMainWindow(parent),
    ui(new Ui::ZooPlotWindow),
    _pgManager{pg}
{
    ui->setupUi(this);


  ui->widgetPlot->setContextMenuPolicy(Qt::CustomContextMenu);
  connect(
              ui->widgetPlot, SIGNAL(customContextMenuRequested(const QPoint&)),
      this, SLOT(myContextMenu(const QPoint&)));



    auto data = pg.getZooPlotData();


    ui->widgetPlot->setAxisTitle(QwtPlot::xBottom, "Mean genuine score");
    ui->widgetPlot->setAxisTitle(QwtPlot::yLeft, "Mean impostor score");
    ui->widgetPlot->setStyleSheet("background-color: white");

    _meanGenuine = data.first;
    _meanImpostor = data.second;



    _zoo.setRawSamples(_meanGenuine.data(), _meanImpostor.data(), _meanGenuine.size());
    _zoo.setStyle(QwtPlotCurve::NoCurve);

    QwtSymbol * symbol = new QwtSymbol(); // XXX QWT takes its ownership
    symbol->setStyle(QwtSymbol::Ellipse);
    symbol->setSize(QSize(7, 7));
    symbol->setBrush(QColor(0,0,255,128));

    _zoo.setSymbol(symbol);

    _zoo.attach(ui->widgetPlot);


    const auto genuineLimits  = getQuartiles(data.first);
    const auto impostorLimits = getQuartiles(data.second);

    const auto genuineMinMax  = std::minmax_element(_meanGenuine.begin(), _meanGenuine.end());
    const auto impostorMinMax  = std::minmax_element(_meanImpostor.begin(), _meanImpostor.end());

    _coords[0].push_back(QPointF(genuineLimits.first, *impostorMinMax.first));
    _coords[0].push_back(QPointF(genuineLimits.first, *impostorMinMax.second));

    _coords[1].push_back(QPointF(genuineLimits.second , *impostorMinMax.first));
    _coords[1].push_back(QPointF(genuineLimits.second, *impostorMinMax.second));



    _coords[2].push_back(QPointF(*genuineMinMax.first, impostorLimits.first));
    _coords[2].push_back(QPointF(*genuineMinMax.second, impostorLimits.first));


    _coords[3].push_back(QPointF(*genuineMinMax.first, impostorLimits.second));
    _coords[3].push_back(QPointF(*genuineMinMax.second, impostorLimits.second));


    for(int i =0; i<4; ++i) {
        _axis[i].setSamples(_coords[i]);
        _axis[i].attach(ui->widgetPlot);
    }



}



void ZooPlotWindow::myContextMenu(const QPoint& p) {

    QMenu myMenu(ui->widgetPlot);
    QAction* itemPDF = myMenu.addAction("Export to PDF");
    myMenu.move(ui->widgetPlot->mapToGlobal(p));

    QAction* selectedItem = myMenu.exec();
    if (selectedItem == itemPDF) {
        QString fname = QFileDialog::getSaveFileName(this,
                                                     "Select file to export ROC curve",
                                                     "",
                                                     "*.pdf");
         if (!fname.isEmpty()) {
            QwtPlotRenderer renderer;
            renderer.renderDocument( ui->widgetPlot, fname, QSizeF( 300, 200 ), 85 );
         }

    }
}




ZooPlotWindow::~ZooPlotWindow() {
    delete ui;
}
