/**
 * Copyright University Bordeaux - CNRS UMR 5800 
 * contributor(s) : Romain Giot (2017)
 * 
 * Romain.Giot@u-bordeaux.fr
 * 
 * Biometric Power Graph (bpg) is a computer program whose purpose is to evaluate biometric authentication
 * systems by displaying with a graph-based representation after having transformed the data.
 * 
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use, 
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * http://www.cecill.info. 
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability. 
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or 
 * data to be ensured and,  more generally, to use and operate it in the 
 * same conditions as regards security. 
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */

#include "bpg/datasetinformation.h"
#include "ui_datasetinformation.h"

#include <iostream>

DatasetInformation::DatasetInformation(QWidget *parent)
: QGroupBox(parent),
  ui(new Ui::DatasetInformation) {
  ui->setupUi(this);

  this->ui->datasetFilename->setReadOnly(true);
  this->ui->datasetNbUsers->setReadOnly(true);
  this->ui->datasetNbSamples->setReadOnly(true);
  this->ui->datasetNbIntraScores->setReadOnly(true);
  this->ui->datasetNbInterScores->setReadOnly(true);
  this->ui->datasetEERValue->setReadOnly(true);
  this->ui->datasetNumberDifferentScores->setReadOnly(true);
  this->ui->datasetEERThreshold->setReadOnly(true);
}

DatasetInformation::~DatasetInformation() {
  delete ui;
}


void DatasetInformation::setFilename(const std::string &fname) {
  this->ui->datasetFilename->setText(QString::fromStdString(fname));
}

void DatasetInformation::setNbUsers(int nb) {
  this->ui->datasetNbUsers->setText(QString::number(nb));
}

void DatasetInformation::setNbSamples(int nb) {
  this->ui->datasetNbSamples->setText(QString::number(nb));
}

void DatasetInformation::setNbIntraScores(int nb) {
  this->ui->datasetNbIntraScores->setText(QString::number(nb));
}

void DatasetInformation::setNbInterScores(int nb) {
  this->ui->datasetNbInterScores->setText(QString::number(nb));
}

void DatasetInformation::setEERVal(double nb) {
  this->ui->datasetEERValue->setText(QString::number(nb));
}

void DatasetInformation::setNbDifferentScores(int nb) {
  this->ui->datasetNumberDifferentScores->setText(QString::number(nb));
}

void DatasetInformation::setEERThr(double nb) {
  this->ui->datasetEERThreshold->setText(QString::number(nb));
}
